% Program for finding growth rates and frequencies of equatorial modes
% driven by cloud-radiation and WISHE. Also finds modes with geostrophic
% and WTG approximations. This script calculates modes but only plots
% eigenvalues, not eigenfunctions. Use 'equaplot.m' to plot both. 
%
% Note: This is SLOW.  May take 30-60 s to execute. 
%
% August, 2019
%
%--------------------------------------------------------------------------
%   Right now, N must be <= 3. For higher N, define Hermite polynomials and
%   derivatives, H, and HD, below. 
%
N=[-1 0 1 2 3];% Mode number. Note: One can add more modes (N>3) but
               % associated Hermite polynomials will have to be added
               % below if eigenfunction plots are desired.
               % 
kmax=15;       % Maximum magnitude of zonal wavenumber to plot (must be integer)                              
n=8;           % Maximum number of roots to plot, in ascending order of growth. (n<=8)
ng=4;          % Same, but for geostrophic modes. (ng<=4)
%
% The parameters below correspond to those in the paper, with Greek letters
% spelled out
%
alpha=1.5;     % Default=1.5
gamma=1;       % Default=1.  This should not be smaller than about 0.2.
G=0.1;         % Default=0.1
C=1.8;         % Default=1.8
D=1.5;         % Default=1.5
chi=1.5;       % Default=1.5
d=0.02;        % Default=0.02
delta=30;      % Default=30
%------------------------------------------------------------------------
close all
clear hp
qn=max(size(N));
k=0:kmax;
qk=max(size(k));
ai=complex(0,1);
%
% Below are factors that are common to all the modes. In the notation of
% equation (9) of the paper: 
%    a1 = f1*sig^2 + f2*sig + f3
%    a2 = f4*sig + f5
%    a3 = f6*sig + f7
%
f1=gamma;  
f2=d.*k.^2-C+chi*gamma;
f3=D*(1+C)+chi*(d.*k.^2-C);
f4=gamma*alpha;
f5=alpha*(d.*k.^2+1);
f6=gamma;
f7=d.*k.^2-C+G*(1+C);
%
% Color specifications for frequency and growth rate plots
%
col=zeros(max(size(qn)),3);
col(1,:)=[0 0 1];
col(2,:)=[0 1 0];
col(3,:)=[1 0 0];
col(4,:)=[0 1 1];
col(5,:)=[1 0.75 0.25];
%
sigsort=zeros(qn,2*qk-1,8);
sigsortg=zeros(qn,2*qk-1,4);
sigsortwtg=zeros(qn,2*qk-1);
hp=zeros(qn,1);
%
% Loop through meridional modes for which N > -1 (The latter are dealt with
%     separately.)
%
for m=2:qn
    sigwtg=-(N(m).*f5+ai.*k.*f7)./(N(m).*f4+ai.*k.*f6); % WTG dispersion relation
    %
    sig=zeros(qk,8);
    b1=zeros(qk,8);
    b2=zeros(qk,8);
    %
    sigg=zeros(qk,4);
    b1g=zeros(qk,4);
    b2g=zeros(qk,4);
    %
    for i=1:qk
        % Here we use MATLAB's equation solver owing to the algebraic
        % complexity. Note that this is pretty slow. 
        syms x;
        eqn=[((N(m)+0.5)^2)*((f4*x+f5(i))^2+4*x*(f1*x^2+f2(i)*x+f3(i))*(f6*x+f7(i))) == ...
            (ai*k(i)*(f6*x+f7(i))-0.5*(f4*x+f5(i))-(x/delta)*(x*(f1*x^2+f2(i)*x+f3(i)) ...
            +ai*k(i)*(f4*x+f5(i))+k(i)^2*(f6*x+f7(i))))^2]; %#ok<NBRAK>
        S=solve(eqn,x);
        sig(i,:)=vpa(S);
        %
        % This loop tests all 8 roots for viability. Non-viable roots set
        % to zero.
        %
        for j=1:8
            a1=f1*sig(i,j)^2+f2(i)*sig(i,j)+f3(i);
            a2=f4*sig(i,j)+f5(i);
            a3=f6*sig(i,j)+f7(i);
            det=sqrt(a2^2+4*a1*sig(i,j)*a3);
            b1(i,j)=(-a2+det)/(4*sig(i,j)*a3);
            b2(i,j)=(-a2-det)/(4*sig(i,j)*a3);
            if N(m) > -1
                fac=(sig(i,j)/delta)*(a1*sig(i,j)+ai*k(i)*a2+a3*k(i)^2);
            else 
                fac=0;
            end    
            res1=ai*k(i)*a3-0.5*a2-(N(m)+0.5)*det-fac;
            res2=ai*k(i)*a3-0.5*a2+(N(m)+0.5)*det-fac;
            %
            if (real(b1(i,j)) <= 0 || (real(b1(i,j)) > 0 && abs(res1) > 1e-4)) && ...
                (real(b2(i,j)) <= 0 || (real(b2(i,j))> 0 && abs(res2) > 1e-4))
                sig(i,j)=-1;
            end  
            if N(m) == 0 && abs(sig(i,j)*a1+ai*k(i)*a2+k(i)^2*a3) < 1e-8 % Handle special case when N=0
                sig(i,j)=-1;
            end    
        end
        %
        % Now calculate 4 roots for geostrophic modes
        %
        A(1)=4*f1.*f6.*(N(m)+0.5).^2; %  A's are coefficients of quartic dispersion relation
        A(2)=4.*(f1.*f7(i)+f2(i).*f6).*(N(m)+0.5).^2;
        A(3)=f4.^2.*N(m)*(N(m)+1)+ai.*k(i).*f4.*f6+f6.^2.*k(i).^2+4.*(f2(i).* ...
            f7(i)+f3(i).*f6).*(N(m)+0.5).^2;
        A(4)=2.*f4.*f5(i).*N(m)*(N(m)+1)+ai.*k(i).*(f4.*f7(i)+f5(i).*f6)+ ...
            2.*k(i).^2.*f6.*f7(i)+4.*f3(i).*f7(i).*(N(m)+0.5).^2;
        A(5)=f5(i).^2.*N(m)*(N(m)+1)+ai.*k(i).*f5(i).*f7(i)+k(i).^2.*f7(i).^2;
        %
        sigg(i,:)=roots(A);   % Note:  This is pretty fast.
        %
        % This loop tests all 4 roots for viability. Non-viable roots set
        % to zero.
        %
        for j=1:4
            a1=f1.*sigg(i,j).^2+f2(i).*sigg(i,j)+f3(i);
            a2=f4.*sigg(i,j)+f5(i);
            a3=f6.*sigg(i,j)+f7(i);
            det=sqrt(a2.^2+4.*a1.*sigg(i,j).*a3);
            b1g(i,j)=(-a2+det)./(4.*sigg(i,j).*a3);
            b2g(i,j)=(-a2-det)./(4.*sigg(i,j).*a3);
            res1=ai.*k(i)*a3-0.5*a2-(N(m)+0.5)*det;
            res2=ai.*k(i)*a3-0.5*a2+(N(m)+0.5)*det;
            %
            if (real(b1g(i,j)) <= 0 || (real(b1g(i,j)) > 0 && abs(res1) > 1e-4)) && ...
                (real(b2g(i,j)) <= 0 || (real(b2g(i,j))> 0 && abs(res2) > 1e-4))
                sigg(i,j)=-1;
            end
        end        
    end   
    %
    % Next remap positive k and two-signed frequency into two-signed
    % frequency and two-signed k. (We only use aboslute value of frequency
    % in plots, though.)
    %
    k2=-max(k):max(k);
    qk2=max(size(k2));
    sig2=zeros(qk2,8);
    sig2g=zeros(qk2,4);
    sigwtg2=zeros(1,qk2);
    for i=1:qk
        for j=1:8
          if imag(sig(i,j)) > 0
              sig2(qk-i+1,j)=sig(i,j);
          else
              sig2(i+qk-1,j)=sig(i,j);
          end
        end
        %
        for j=1:4
          if imag(sigg(i,j)) > 0
              sig2g(qk-i+1,j)=sigg(i,j);
          else
              sig2g(i+qk-1,j)=sigg(i,j);
          end
        end
        %
        if imag(sigwtg(i)) > 0
          sigwtg2(qk-i+1)=sigwtg(i);
        else
          sigwtg2(i+qk-1)=sigwtg(i);
        end  
    end    
    %
    % Sort roots by real part of growth rate
    %
    for i=1:qk2
        sigsort(m,i,:)=sort(sig2(i,:),'descend','ComparisonMethod','real');
        sigsortg(m,i,:)=sort(sig2g(i,:),'descend','ComparisonMethod','real');
        sigsortwtg(m,i)=sigwtg2(i);
    end 
end    
%
% Treat v=0 (N=-1) mode seperately
%
p(1)=f1;
for i=1:qk 
    p(2)=f2(i);
    p(3)=f3(i)+f6.*k(i)^2+ai*k(i)*f4;
    p(4)=f7(i)*k(i)^2+ai*k(i)*f5(i);
    sig=roots(p);
    sig(imag(sig)>0)=0; % Roots must have negative imaginary part fo satisfy BCs
    sigsort(1,i+qk-1,1:3)=sort(sig,'descend','ComparisonMethod','real');
end
%
sigsortg(1,:,1:3)=sigsort(1,:,1:3);
sigsortwtg(1,qk:qk2)=-(-1*f5+ai.*k.*f7)./(-1*f4+ai.*k.*f6); % WTG dispersion relation
%
% For plotting, set zero (including non-viable) roots to NaNs
%
sigsort(sigsort==-1)=NaN.*(1+ai);
sigsortg(sigsortg==-1)=NaN.*(1+ai);
sigsortwtg(sigsortwtg==-1)=NaN.*(1+ai);
%
msizefac(1,1)=60./max(max(real(sigsort(:,:,1)))); % Set marker size factor according to fastest growing mode
%
figure('Position',[50 275 600 450]);
hold on
%
% The code below is just to fool MATLAB into producing a nice legend later
%
for m=1:qn
    [~,imax]=max(real(sigsort(m,:,1)));
    u=plot(k2(imax),abs(imag(sigsort(m,imax,1))),'.','color',col(m,:),'markersize',10);
    hp(m,:)=u;
end
%
%  End of code for legend; next produce eignevalue plots
%
for m=1:qn
    for i=1:qk2  % Plot frequencies with marker sizes proportional to growth rates
        for ni=1:n
            msize=round(msizefac*real(sigsort(m,i,ni)));
            msize=max(msize,10);
            if ~isnan(sigsort(m,i,ni)) && real(sigsort(m,i,ni)) >= 0 && abs(imag(sigsort(m,i,ni))) >= 1e-4
                plot(k2(i),(abs(imag(sigsort(m,i,ni)))),'.','color',col(m,:),'markersize',msize)
            end
        end
    end
end
hold off
set(gca,'fontsize',12,'fontweight','bold')
xlabel('Zonal Wavenumber')
ylabel('Frequency')
title('Full eigenvalue spectrum')
yy=get(gca,'ylim');
ymax=yy(2);
ypos=0.9*ymax;
uu=max(max(real(sigsort(:,:,1))));
uy=0.01*(round(100*uu));
tstr=['MAX(real(\sigma)) = ',num2str(uy)];
text(-5, ypos, tstr,'fontsize',11,'fontweight','bold')
legend(hp,{'n=-1','n=0','n=1','n=2','n=3';},'location','best','AutoUpdate','off');
savefig('figtemp')
%
% Create copy of dispersion relation figure excluding higher frequency modes
%
figure('Position',[700 275 600 450]);
hold on
for m=1:qn
    [~,imax]=max(real(sigsort(m,:,1)));
    if real(sigsort(m,imax,1)) >= 0 && abs(imag(sigsort(m,imax,1))) >= 1e-4
        u=plot(k2(imax),abs(imag(sigsort(m,imax,1))),'.','color',col(m,:),'markersize',18);
    end    
    hp(m,:)=u;
    for i=1:qk2  % Plot frequencies with marker sizes proportional to growth rates
        for ni=1:8
            msize=round(msizefac*real(sigsort(m,i,ni)));
            msize=max(msize,10);
            if real(sigsort(m,i,ni)) >= 0 && abs(imag(sigsort(m,i,ni))) >= 1e-4
                plot(k2(i),(abs(imag(sigsort(m,i,ni)))),'.','color',col(m,:),'markersize',msize)
            end    
        end
    end
end
hold off
set(gca,'fontsize',12,'fontweight','bold')
set(gca,'ylim',[0 1.5])
xlabel('Zonal Wavenumber')
ylabel('Frequency')
yy=get(gca,'ylim');
ymax=yy(2);
ypos=0.9*ymax;
xx=get(gca,'xlim');
xpos=xx(1)+0.3*(xx(2)-xx(1));
text(xpos, ypos, tstr,'fontsize',11,'fontweight','bold')
legend(hp,{'n=-1','n=0','n=1','n=2','n=3';},'location','best','AutoUpdate','off');
title('Low frequency only')
savefig('figtemplow')
%
%  Save various quantities for use in plotting script.
%
save equatemp sigsort sigsortg sigsortwtg msizefac N k2 k ai delta chi ...
    alpha C f1 f2 f3 f4 f5 f6 f7 col G d gamma
